@@ -1,7 +1,9 @@
+import argparse
+import json
 import os
+import platform
 import re
 import sys
-import platform
 
 from qisys import ui
 import qisys.command
@@ -11,7 +13,15 @@ import qibuild.build
 import qibuild.gdb
 import qibuild.dylibs
 import qibuild.dlls
+import qibuild.test_runner
 import qitoolchain.toolchain
+import qitest.conf
+
+
+def read_install_manifest(filepath, rootdir):
+    with open(filepath, "r") as f:
+        return [filename.strip() for filename in f.readlines()]
+
 
 class BuildProject(object):
     def __init__(self, build_worktree, worktree_project):
@@ -41,34 +51,16 @@ class BuildProject(object):
         the build profiles, and the build type (debug/release)
 
         """
-        parts = ["build"]
-        toolchain = self.build_config.toolchain
-        build_type = self.build_config.build_type
-        visual_studio = self.build_config.using_visual_studio
-        if toolchain:
-            parts.append(toolchain.name)
-        else:
-            parts.append("sys-%s-%s" % (platform.system().lower(),
-                                        platform.machine().lower()))
-        profiles = self.build_config.profiles
-        for profile in profiles:
-            parts.append(profile)
-
-        # When using cmake + visual studio, sharing the same build dir with
-        # several build config is mandatory.
-        # Otherwise, it's not a good idea, so we always specify it
-        # when it's not "Debug" (the default)
-        if not visual_studio:
-            if build_type and build_type != "Debug":
-                parts.append(build_type.lower())
-
-        # FIXME: handle custom build dir
-        return os.path.join(self.path, "-".join(parts))
+        return os.path.join(self.path, self.build_config.build_directory())
 
     @property
     def cmake_cache(self):
         return os.path.join(self.build_directory, "CMakeCache.txt")
 
+    @property
+    def qitest_json(self):
+        return os.path.join(self.build_directory, "qitest.json")
+
     @property
     def cmake_args(self):
         """ The list of CMake arguments to use when configuring the
@@ -93,7 +85,7 @@ class BuildProject(object):
 
     @property
     def cmake_generator(self):
-        return self.build_config.cmake_generator
+        return qibuild.cmake.get_cached_var(self.build_directory, "CMAKE_GENERATOR")
 
     @property
     def using_visual_studio(self):
@@ -102,6 +94,10 @@ class BuildProject(object):
     def using_make(self):
         return self.build_config.using_make
 
+    @property
+    def verbose_make(self):
+        return self.build_config.verbose_make
+
     def write_qi_path_conf(self, sdk_dirs):
         """ Write the <build>/sdk/share/qi/path.conf file. This file
         can be used for instance by qi::path::find() functions, to
@@ -205,11 +201,54 @@ set(QIBUILD_PYTHON_PATH "%s" CACHE STRING "" FORCE)
         try:
             qibuild.cmake.cmake(self.path, self.build_directory,
                                 cmake_args, env=self.build_env, **kwargs)
-        except qisys.command.CommandFailedException:
-            raise qibuild.build.ConfigureFailed(self)
+        except qisys.command.CommandFailedException as error:
+            raise qibuild.build.ConfigureFailed(self, error)
+        # Write the qitest.json file:
+        tests = self.parse_qitest_cmake()
+        with open(self.qitest_json, "w") as fp:
+            json.dump(tests, fp, indent=2)
+
+    def parse_qitest_cmake(self):
+        """ The qitest.cmake is written from CMake """
+        qitest_cmake_path = os.path.join(self.build_directory, "qitest.cmake")
+        tests = list()
+        if not os.path.exists(qitest_cmake_path):
+            return list()
+        with open(qitest_cmake_path, "r") as fp:
+            lines = fp.readlines()
+        parser = argparse.ArgumentParser()
+        parser.add_argument("cmd", nargs="+")
+        parser.add_argument("--name", required=True)
+        parser.add_argument("--gtest", action="store_true",
+                            help="Tell qitest this is a test using gtest")
+        parser.add_argument("--timeout", type=int)
+        parser.add_argument("--nightly", action="store_true")
+        parser.add_argument("--perf", action="store_true")
+        parser.add_argument("--working-directory")
+        parser.add_argument("--env", action="append",
+                            dest="environment")
+        parser.set_defaults(nightly=False, perf=False)
+        def log_error(message, line):
+            test_name = line.split(";")[1]
+            mess = "Could not parse test options for test: '%s'\n" % test_name
+            mess += "Error was: %s" % message
+            ui.error(mess)
+        for line in lines:
+            parser.error = lambda message : log_error(message, line)
+            line = line.strip()
+            try:
+                args = parser.parse_args(args=line.split(";"))
+            except SystemExit:
+                break
+            test = vars(args)
+            as_list = test["environment"]
+            if as_list:
+                test["environment"] = dict(x.split("=") for x in as_list)
+            tests.append(test)
+        return tests
 
     def build(self, num_jobs=None, rebuild=False, target=None,
-              verbose_make=False, coverity=False, env=None):
+              coverity=False, env=None):
         """ Build the project """
         timer = ui.timer("make %s" % self.name)
         timer.start()
@@ -238,11 +277,12 @@ set(QIBUILD_PYTHON_PATH "%s" CACHE STRING "" FORCE)
             build_env = self.build_env.copy()
         else:
             build_env = env
-        if verbose_make:
-            if "Makefiles" in self.cmake_generator:
-                build_env["VERBOSE"] = "1"
-            if self.cmake_generator == "Ninja":
-                cmd.append("-v")
+        if self.verbose_make:
+            if self.cmake_generator:
+                if "Makefiles" in self.cmake_generator:
+                    build_env["VERBOSE"] = "1"
+                if self.cmake_generator == "Ninja":
+                    cmd.append("-v")
         try:
             qisys.command.call(cmd, env=build_env)
         except qisys.command.CommandFailedException:
@@ -290,6 +330,9 @@ set(QIBUILD_PYTHON_PATH "%s" CACHE STRING "" FORCE)
             useful for `qibuild deploy`
 
         """
+        installed = list()
+        if components is None:
+            components = list()
         # DESTDIR=/tmp/foo and CMAKE_PREFIX="/usr/local" means
         # dest = /tmp/foo/usr/local
         destdir = qisys.sh.to_native_path(destdir)
@@ -316,12 +359,18 @@ set(QIBUILD_PYTHON_PATH "%s" CACHE STRING "" FORCE)
             self.build(target="preinstall", num_jobs=num_jobs, env=build_env)
         if components:
             for component in components:
-                self._install_component(destdir, component)
+                files = self._install_component(destdir, component)
+                installed.extend(files)
         else:
             self.build(target="install", env=build_env)
+            manifest_path = os.path.join(self.build_directory, "install_manifest.txt")
+            installed.extend(read_install_manifest(manifest_path, destdir))
+        if "test" in components:
+            self._install_qitest_json(destdir)
 
         if split_debug:
             self.split_debug(destdir)
+        return installed
 
     def _install_component(self, destdir, component):
         build_env = self.build_env.copy()
@@ -334,22 +383,32 @@ set(QIBUILD_PYTHON_PATH "%s" CACHE STRING "" FORCE)
         ui.debug("Installing", component)
         qisys.command.call(["cmake"] + cmake_args, cwd=self.build_directory,
                             env=build_env)
+        manifest_path = os.path.join(self.build_directory, "install_manifest_%s.txt" % component)
+        installed = read_install_manifest(manifest_path, destdir)
+        return installed
 
-    def deploy(self, url, port=22, split_debug=False,
-               use_rsync=True, with_tests=True):
-        """ Deploy the project to a remote url """
-        destdir = os.path.join(self.build_directory, "deploy")
-        #create folder for project without install rules
-        qisys.sh.mkdir(destdir, recursive=True)
-        components=["runtime"]
-        if with_tests:
-            components.append("test")
-        self.install(destdir, components=components)
-        if split_debug:
-            self.split_debug(destdir)
-        ui.info(ui.green, "Sending binaries to target ...")
-        qibuild.deploy.deploy(destdir, url, use_rsync=use_rsync, port=port)
+    def _install_qitest_json(self, destdir):
+        if not os.path.exists(self.qitest_json):
+            return
+        tests = qitest.conf.parse_tests(self.qitest_json)
+        tests = qitest.conf.relocate_tests(self, tests)
+        qitest.conf.write_tests(tests, os.path.join(destdir, "qitest.json"))
 
+    def run_tests(self, **kwargs):
+        """ Run the tests for this project """
+        ui.info(ui.green, "Testing", self.name, "...")
+        test_runner = qibuild.test_runner.ProjectTestRunner(self)
+        test_runner.cwd = os.path.join(self.sdk_directory, "bin")
+        test_runner.env = self.build_env
+        test_runner.pattern = kwargs.get("pattern")
+        test_runner.perf = kwargs.get("perf", False)
+        test_runner.nightly = kwargs.get("nightly", False)
+        test_runner.coverage = kwargs.get("coverage")
+        test_runner.valgrind = kwargs.get("valgrind")
+        test_runner.verbose = kwargs.get("verbose_tests")
+        test_runner.num_cpus = kwargs.get("num_cpus", -1)
+        test_runner.num_jobs = kwargs.get("num_jobs", 1)
+        return test_runner.run()
 
     def fix_shared_libs(self, paths):
         """ Do some magic so that shared libraries from other projects and
